// /file PostfixMilterConf.cpp
//
// Copyright (C) 2011, ARM Research Labs, LLC.
// See www.armresearch.com for the copyright terms.
//
// This file contains the functions for PostfixMilterConf.
//
// $Id$
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#include <exception>
#include <stdexcept>
#include <sstream>

#include "Utility.hpp"
#include "PostfixMilterConf.hpp"

//////////////////////////////////////////////////////////////////////////////////////////////////////////
// Configuration. ////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////

/// SNFMilter socket specification.
const std::string SnfMilterSocketSpec("unix:/var/snf-milter/socket");

/// Milter keyword in the postfix main.cf file.
const std::string SmtpdMilterKeyword("smtpd_milters");

//////////////////////////////////////////////////////////////////////////////////////////////////////////
// End of configuration. /////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////

void
PostfixMilterConf::ConfLine(std::string Line) {

    ConfigurationLine = Utility::Trim(Line);                                    // Remove leading and trailing whitespace.

}

bool
PostfixMilterConf::IsIntegrated() {

    return (IsMilterLine() && ContainsSnfMilterSocketSpec());

}

void
PostfixMilterConf::AddIntegration() {

    if (IsIntegrated()) {

	return;

    }

    if (IsMilterLine()) {                                                       // Add to existing milter line.

	std::string NewConfLine;

	NewConfLine = SmtpdMilterKeyword + " =";

	// Skip to "=" in configuration line.
	std::string::size_type NextIndex;

	NextIndex = ConfigurationLine.find("=");

        if (std::string::npos == NextIndex) {                                   // Check format.

	    std::ostringstream Temp;

	    Temp << "Error processing postfix main.cf file smtpd_milters line:  '"
		 << ConfigurationLine << "'";

	    throw std::runtime_error(Temp.str());

	}

	std::string ExistingMilters;

	ExistingMilters = Utility::Trim(ConfigurationLine.substr(NextIndex + 1)); // Should contain existing milters.

	while (ExistingMilters != "") {

	    NewConfLine += " ";

	    std::string NextMilter;

	    NextIndex = ExistingMilters.find_first_of(" ,");
	    NextMilter = Utility::Trim(ExistingMilters.substr(0, NextIndex));
	    ExistingMilters = Utility::Trim(ExistingMilters.substr(NextIndex + 1));

	    if (NextMilter == "") {

		std::ostringstream Temp;

		Temp << "Error processing postfix main.cf file smtpd_milters line:  '"
		     << ConfigurationLine << "'";

		throw std::runtime_error(Temp.str());

	    }

	    NewConfLine += NextMilter;

	}

	ConfigurationLine = NewConfLine + " ";
	ConfigurationLine += SnfMilterSocketSpec;

    } else if (ConfigurationLine == "") {                                       // Empty configuration line.

	ConfigurationLine = SmtpdMilterKeyword + " = ";
	ConfigurationLine += SnfMilterSocketSpec;

    } else {                                                                    // Unexpected non-empty line.

	std::ostringstream Temp;

	Temp << "Internal error:  Attempted to modify a line in main.cf that does not begin with '"
	     << SmtpdMilterKeyword << "'";

	throw std::runtime_error(Temp.str());

    }

}

void
PostfixMilterConf::RemoveIntegration() {

    if (!IsIntegrated()) {

	return;

    }

    if (IsMilterLine()) {                                                       // Remove from an existing milter line.

	std::string NewConfLine;

	NewConfLine = SmtpdMilterKeyword + " =";

	// Skip to "=" in configuration line.
	std::string::size_type NextIndex;

	NextIndex = ConfigurationLine.find("=");

        if (std::string::npos == NextIndex) {                                   // Check format.

	    std::ostringstream Temp;

	    Temp << "Error processing postfix main.cf file smtpd_milters line:  '"
		 << ConfigurationLine << "'";

	    throw std::runtime_error(Temp.str());

	}

	std::string ExistingMilters;
	bool AddedMilter = false;

	ExistingMilters = Utility::Trim(ConfigurationLine.substr(NextIndex + 1)); // Should contain existing milters.

	while (ExistingMilters != "") {

	    NewConfLine += " ";

	    std::string NextMilter;

	    NextIndex = ExistingMilters.find_first_of(" ,");

	    if (std::string::npos == NextIndex) {

		NextMilter = ExistingMilters;
		ExistingMilters = "";

	    } else {

		NextMilter = Utility::Trim(ExistingMilters.substr(0, NextIndex));
		ExistingMilters = Utility::Trim(ExistingMilters.substr(NextIndex + 1));

	    }

	    if (NextMilter == "") {

		std::ostringstream Temp;

		Temp << "Error processing postfix main.cf file smtpd_milters line:  '"
		     << ConfigurationLine << "'";

		throw std::runtime_error(Temp.str());

	    }

            if (NextMilter != SnfMilterSocketSpec) {                            // Copy if not for SNFMilter.

		NewConfLine += NextMilter;
		AddedMilter = true;

	    }

	}

	if (AddedMilter) {

	    ConfigurationLine = NewConfLine;

	} else {

	    ConfigurationLine = "";

	}

    } else {                                                                    // Unexpected non-empty line.

	std::ostringstream Temp;

	Temp << "Internal error:  Attempted to modify a line in main.cf that does not begin with '"
	     << SmtpdMilterKeyword << "'";

	throw std::runtime_error(Temp.str());

    }

}

std::string
PostfixMilterConf::ConfLine() {

    return ConfigurationLine;

}

bool
PostfixMilterConf::IsMilterLine() {

    return (ConfigurationLine.find(SmtpdMilterKeyword) == 0);

}

bool
PostfixMilterConf::ContainsSnfMilterSocketSpec() {

    return (ConfigurationLine.find(SnfMilterSocketSpec) != std::string::npos);

}
